{% extends 'cfblog_base.html' %}

{% block head %}
    <title data-cms-content="title">Page Title</title>
    <meta data-cms-attr="content:decrp" name="description" content="default content" />
{% endblock %}

{% block content %}
    <span data-cms-attr="id:body_attr" id="random">default body</span>
    <div data-cms-content="md:body" class="random">
    </div>
{% endblock %}
